#include	<stdio.h>
#include	<time.h>

#include	"tinyda.h"


int		EnumCall( const char *key, void *rec, void *arg )
{
	// レコードが 0 でなければ終了
	if ( *( int * )rec )
	{
		printf( "%d\t%s\n", *( int * )rec, key );
		return	*( int * )rec;
	}

	return	DA_OK;
}

int		main( int argc, char *argv[] )
{
	int		i, ret;
	int		*val, idx;
	DAStat	stat;
	hDASet	set;
	hDADic	dic;
	clock_t		cl;
	const char	*s, *start, *input = "motor";

	// キー集合を作成
	if ( ( ret = DASet_Open( &set ) ) != DA_OK )
		fprintf( stderr, "DASet_Open(): %d\n", ret );

	// キー集合をファイルで入力
	cl = clock();
	for ( i = 1; i < argc; i++ )
	{
		// タブ，改行による区切り
		ret = DASet_Load( set, argv[i], "\t\r\n", sizeof( int ) );
		if ( ret != DA_OK )
			fprintf( stderr, "DADic_Load(): %s: %d\n", argv[i], ret );
	}
	cl = clock() - cl;
	printf( "Load: %.3f sec\n", 1.0 * cl / CLOCKS_PER_SEC );

	// キー集合にキーを追加
//	if ( ( ret = DASet_Insert( set, "Tes", sizeof( int ) ) ) != DA_OK )
//		fprintf( stderr, "DADic_Insert(): %d\n", ret );
//	if ( ( ret = DASet_Insert( set, "Test", sizeof( int ) ) ) != DA_OK )
//		fprintf( stderr, "DADic_Insert(): %d\n", ret );
//	if ( ( ret = DASet_Insert( set, "Test-Key", sizeof( int ) ) ) != DA_OK )
//		fprintf( stderr, "DADic_Insert(): %d\n", ret );

	// 辞書を構築
	cl = clock();
	if ( ( ret = DADic_Build( &dic, set, sizeof( int ) ) ) != DA_OK )
		fprintf( stderr, "DADic_Build(): %d\n", ret );
	cl = clock() - cl;
	printf( "Build: %.3f sec\n", 1.0 * cl / CLOCKS_PER_SEC );

	// キー集合を破棄
	if ( ( ret = DASet_Close( set ) ) != DA_OK )
		fprintf( stderr, "DASet_Close(): %d\n", ret );

	// 辞書の情報を出力
	if ( ( ret = DADic_Status( dic, &stat ) ) != DA_OK )
		fprintf( stderr, "DADic_Stat(): %d\n", ret );
	printf( "Size: 4 * %d + %d = %d\n", stat.nbc, stat.ntail, stat.size );
	printf( "Key: %d\n", stat.nkey );
	printf( "Max-len: %d\n", stat.maxlen );
	printf( "Boundary: %d\n", stat.boundary );

	// キーの検索とレコードの設定
	if ( ( val = DADic_Search( dic, "pen" ) ) == NULL )
		fprintf( stderr, "DADic_Search(): Not found\n" );
	else
		*val = 100;

	// 辞書を出力
	if ( ( ret = DADic_Save( dic, "test.dic" ) ) != DA_OK )
		fprintf( stderr, "DADic_Save(): %d\n", ret );

	// 辞書を破棄
	if ( ( ret = DADic_Close( dic ) ) != DA_OK )
		fprintf( stderr, "DADic_Close(): %d\n", ret );

	// 辞書を入力
	if ( ( ret = DADic_Open( &dic, "test.dic" ) ) != DA_OK )
		fprintf( stderr, "DADic_Open(): %d\n", ret );

	// 前方一致検索
	idx = 0;
	start = s = input;
	while ( ( val = DADic_Prefix( dic, &s, &idx ) ) != NULL )
		printf( "DADic_Prefix(): [%.*s]%s: %d\n", s - start, start, s, *val );

	// 部分一致検索
	idx = 0;
	start = s = input;
	while ( ( val = DADic_Infix( dic, &start, &s, &idx ) ) != NULL )
		printf( "DADic_Infix(): %.*s[%.*s]%s: %d\n",
				start - input, input, s - start, start, s, *val );

	// 登録キーの列挙
	cl = clock();
	if ( ( ret = DADic_Enum( dic, NULL, EnumCall ) ) != DA_OK )
		fprintf( stderr, "DADic_Enum(): %d\n", ret );
	cl = clock() - cl;
	printf( "Enum: %.3f sec\n", 1.0 * cl / CLOCKS_PER_SEC );

	// 辞書を破棄
	if ( ( ret = DADic_Close( dic ) ) != DA_OK )
		fprintf( stderr, "DADic_Close(): %d\n", ret );

	return	0;
}
