#ifndef		__TINY_DOUBLE_ARRAY_H_
#define		__TINY_DOUBLE_ARRAY_H_


// エラーコード
#define		DA_OK		0		// 成功
#define		DA_EHDL		( -1 )	// 不正なハンドル
#define		DA_EMEM		( -2 )	// メモリエラー
#define		DA_EIO		( -3 )	// 入出力エラー
#define		DA_EARG		( -4 )	// 不正な引数
#define		DA_EFMT		( -5 )	// 辞書の書式エラー
#define		DA_EMAX		( -6 )	// サイズ上限
#define		DA_EVER		( -7 )	// バージョンエラー


#ifdef	__cplusplus
extern	"C"	{
#endif


// キー集合のハンドル
typedef		struct DASet	*hDASet;

// 辞書のハンドル
typedef		struct DADic	*hDADic;

// 辞書の状態
typedef		struct DAStat
{
	int		nbc;		// 配列 BC の長さ
	int		ntail;		// 配列 TAIL の長さ
	int		size;		// 配列 BC と配列 TAIL に割り当てたバイト数
	int		nkey;		// キー数
	int		maxlen;		// 最大キー長 ( NULL 文字を除く )
	int		boundary;	// バイト境界
}	DAStat, *hDAStat;

// 列挙用の関数
// 引数は，キー，レコード，オリジナルの順
// 返り値を DA_OK 以外にすると列挙を終了する
typedef		int ( *fDACall )( const char *, void *, void * );


// キー集合の作成
int		DASet_Open( hDASet *h );
// キー集合の読み込み
// タブや改行による区切りを delim で指定する
// レコードのサイズを rec で指定する
int		DASet_Load( hDASet h, const char *path, const char *delim, int rec );
// キーの追加
// レコードのサイズを rec で指定する
int		DASet_Insert( hDASet h, const char *key, int rec );
// キー集合の破棄
int		DASet_Close( hDASet h );

// 辞書の作成
// レコードは boundary で指定したバイト境界に配置される
int		DADic_Build( hDADic *h, hDASet set, int boundary );
// 辞書の読み込み
int		DADic_Open( hDADic *h, const char *path );
// 辞書の破棄
int		DADic_Close( hDADic h );
// 辞書の保存
int		DADic_Save( hDADic h, const char *path );

// 辞書情報の取得
int		DADic_Status( hDADic h, hDAStat st );
// 登録キーを昇順に列挙
int		DADic_Enum( hDADic h, void *arg, fDACall call );

// 入力文字列と一致するキーを辞書から検索
// レコードへのポインタを返す
void	*DADic_Search( hDADic h, const char *key );
// 入力文字列の前方部分列と一致するキーを辞書から検索
// レコードへのポインタを返す
// 最初はインデックスを 0 にしておき，NULL を返すまで繰り返し呼び出す
void	*DADic_Prefix( hDADic h, const char **s, int *idx );
// 入力文字列に出現するキーを辞書から検索
// レコードへのポインタを返す
// 最初の呼び出しでは，start = s，インデックスを 0 にしておく
// NULL を返すまで繰り返し呼び出す
void	*DADic_Infix( hDADic h, const char **start, const char **s, int *idx );

// エラーメッセージ
const char	*DAErr_String( int err );


#ifdef	__cplusplus
}
#endif


#endif
